source("clustercompare.R")
library("ecp")
library("mvtnorm")

for (S in c(100, 200, 300)) {
  set.seed(2012)
  er1 <- er2 <- er3 <- matrix(0, nrow = 5, ncol = 1000)
  correct <- rep(c(1, 2, 3), rep(S, 3))

  M <- c(0, 0)
  M1 <- c(1, 1)
  M2 <- c(2, 2)
  M3 <- c(3, 3)
  cr <- diag(2)
  
  for (i in 1:1000) {
    X1 <- rbind(rmvnorm(S, M, cr), rmvnorm(S, M1, cr), rmvnorm(S, M, cr))
    X2 <- rbind(rmvnorm(S, M, cr), rmvnorm(S, M2, cr), rmvnorm(S, M, cr))
    X3 <- rbind(rmvnorm(S, M, cr), rmvnorm(S, M3, cr), rmvnorm(S, M, cr))
    
    e1 <- e.divisive(X1, R = 499)
    e2 <- e.divisive(X2, R = 499)
    e3 <- e.divisive(X3, R = 499)
    
    er1[, i] <- compareCluster(correct, e1$cluster)
    er2[, i] <- compareCluster(correct, e2$cluster)
    er3[, i] <- compareCluster(correct, e3$cluster)
  }

  ## Obtain average Rand Values for each test case as well as standard errors
  ERandAvg1 <- mean(er1[1, ])
  ERandSE1 <- sd(er1[1, ])/sqrt(1000)
  ERandAvg2 <- mean(er2[1, ])
  ERandSE2 <- sd(er2[1, ])/sqrt(1000)
  ERandAvg3 <- mean(er3[1, ])
  ERandSE3 <- sd(er3[1, ])/sqrt(1000)
  
  result <- matrix(0, nrow = 3, ncol = 3)
  colnames(result) <- c("mu", "Average Rand", "Standard Error")
  
  result[, 1] <- c(1, 2, 3)
  result[, 2] <- c(ERandAvg1, ERandAvg2, ERandAvg3)
  result[, 3] <- c(ERandSE1, ERandSE2, ERandSE3)

  print(result)
}
