######## e.divisive() examples ########

########
# Section 3.1.1, Change in univariate normal distribution
set.seed(250)
library("ecp")
period1 <- rnorm(100)
period2 <- rnorm(100, 0, 3)
period3 <- rnorm(100, 2, 1)
period4 <- rnorm(100, 2, 4)
X.3.1.1 <- matrix(c(period1, period2, period3, period4), ncol = 1)
output1 <- e.divisive(X.3.1.1, R = 499, alpha = 1)
output2 <- e.divisive(X.3.1.1, R = 499, alpha = 2)
output2$estimates
output1$k.hat
output1$order.found
output1$estimates
output1$considered.last
output1$p.values
output1$permutations
# Make plot
ts.plot(X.3.1.1, ylab = "Value", main = "Change in Univariate Gaussian Sequence")
abline(v = c(101, 201, 301), col = "blue")
abline(v = output1$estimates[c(-1, -5)], col = "red", lty = 2)

########
# Section 3.1.2, Multivariate change in covariance
set.seed(200)
library("ecp")
library("mvtnorm")
mu <- rep(0, 3)
covA <- matrix(c(1, 0, 0, 0, 1, 0, 0, 0, 1), 3, 3)
covB <- matrix(c(1, 0.9, 0.9, 0.9, 1, 0.9, 0.9, 0.9, 1), 3, 3)
period1 <- rmvnorm(250, mu, covA)
period2 <- rmvnorm(250, mu, covB)
period3 <- rmvnorm(250, mu, covA)
X.3.1.2 <- rbind(period1, period2, period3)
output <- e.divisive(X.3.1.2, R = 499, alpha = 1)
output$estimates


########
# Section 3.1.3, Multivariate change in tails
set.seed(100)
library("ecp")
library("mvtnorm")
mu <- rep(0, 2)
period1 <- rmvnorm(250, mu, diag(2))
period2 <- rmvt(250, sigma = diag(2), df = 2)
period3 <- rmvnorm(250, mu, diag(2))
X.3.1.3 <- rbind(period1, period2, period3)
output <- e.divisive(X.3.1.3, R = 499, alpha = 1)
output$estimates
# Make plot
par(mfrow = c(1, 3))
plot(X.3.1.3[1:250, ], main = "Period 1", xlim = c(-10, 10), ylim = c(-25, 10), xlab = "", 
  ylab = "", cex.main = 2)
plot(X.3.1.3[251:500, ], main = "Period 2", xlim = c(-10, 10), ylim = c(-25, 10), 
  xlab = "", ylab = "", cex.main = 2)
plot(X.3.1.3[501:750, ], main = "Period 3", xlim = c(-10, 10), ylim = c(-25, 10), 
  xlab = "", ylab = "", cex.main = 2)



######## e.agglo() examples ########

########
# Section 4.1.1, Change in normal distributions
library("ecp")
member <- rep(1:40, rep(10, 40))
output <- e.agglo(X = X.3.1.1, member = member, alpha = 1)
output$estimates
tail(output$fit, 5)
output$progression[1, 1:10]
output$merged[1:4, ]


########
# Section 4.1.2, Multivariate chagne in covariance
library("ecp")
member <- rep(1:15, rep(50, 15))
pen <- function(x) {
  -length(x)
}
output1 <- e.agglo(X = X.3.1.2, member = member, alpha = 1)
output2 <- e.agglo(X = X.3.1.2, member = member, alpha = 1, penalty = pen)
output1$estimates
output2$estimates

###########
# Spatio-temporal point process example
# Create the true density plots
library("mvtnorm")
m1 <- rep(-7, 2)
m2 <- rep(0, 2)
m3 <- c(5.5, 0)
c1 <- 25 * diag(2)
c2 <- matrix(c(9, 0, 0, 1), 2)
c3 <- matrix(c(9, 0.9, 0.9, 9), 2)

w1 <- rep(1/3, 3)
w2 <- c(1/5, 1/2, 3/10)
w3 <- c(7/20, 3/10, 7/20)
w4 <- c(1/5, 3/10, 1/2)

f <- function(p, w) {
  a <- dmvnorm(p, m1, c1)
  b <- dmvnorm(p, m2, c2)
  d <- dmvnorm(p, m3, c3)
  k <- c(a, b, d) * w
  return(sum(k))
}

d1 <- d2 <- d3 <- d4 <- list()
d1$z <- d2$z <- d3$z <- d4$z <- matrix(0, 250, 250)
d1$x <- d2$x <- d3$x <- d4$x <- seq(from = -15, to = 15, length.out = 250)
d1$y <- d2$y <- d3$y <- d4$y <- seq(from = -15, to = 15, length.out = 250)

# Obtain density values.

for (i in 1:250) {
  for (j in 1:250) {
    d1$z[i, j] <- f(c(d1$x[i], d1$y[j]), w1)
    d2$z[i, j] <- f(c(d2$x[i], d2$y[j]), w2)
    d3$z[i, j] <- f(c(d3$x[i], d3$y[j]), w3)
    d4$z[i, j] <- f(c(d4$x[i], d4$y[j]), w4)
  }
}

# Generate plots.
par(mfrow = c(2, 2))

image(d1, main = "Density for period 1", cex.main = 1.4, cex.axis = 0.8, col = RColorBrewer::brewer.pal(9, 
  "PuBuGn"))
image(d2, main = "Density for period 2", cex.main = 1.4, cex.axis = 0.8, col = RColorBrewer::brewer.pal(9, 
  "PuBuGn"))
image(d3, main = "Density for period 3", cex.main = 1.4, cex.axis = 0.8, col = RColorBrewer::brewer.pal(9, 
  "PuBuGn"))
image(d4, main = "Density for period 4", cex.main = 1.4, cex.axis = 0.8, col = RColorBrewer::brewer.pal(9, 
  "PuBuGn"))


# Perform change point analysis and create estimated densities

library("mvtnorm")
library("combinat")
library("MASS")
library("ecp")
set.seed(2013)
lambda <- 1500  # This is the overall arrival rate per unit time.
# set of distribution means
muA <- c(-7, -7)
muB <- c(0, 0)
muC <- c(5.5, 0)
# set of distribution covariance matrices
covA <- 25 * diag(2)
covB <- matrix(c(9, 0, 0, 1), 2)
covC <- matrix(c(9, 0.9, 0.9, 9), 2)
# time intervals
time.interval <- matrix(c(0, 1, 3, 4.5, 1, 3, 4.5, 7), 4, 2)
# mixing coefficents
mixing.coef <- rbind(c(1/3, 1/3, 1/3), c(0.2, 0.5, 0.3), c(0.35, 0.3, 0.35), c(0.2, 
  0.3, 0.5))
stppData <- NULL
for (i in 1:4) {
  count <- rpois(1, lambda * diff(time.interval[i, ]))
  Z <- rmultz2(n = count, p = mixing.coef[i, ])
  S <- rbind(rmvnorm(Z[1], muA, covA), rmvnorm(Z[2], muB, covB), rmvnorm(Z[3], 
    muC, covC))
  X <- cbind(rep(i, count), runif(n = count, time.interval[i, 1], time.interval[i, 
    2]), S)
  stppData <- rbind(stppData, X[order(X[, 2]), ])
}
member <- as.numeric(cut(stppData[, 2], breaks = seq(0, 7, by = 1/12)))
output <- e.agglo(X = stppData[, 3:4], member = member, alpha = 1)

# Obtain start (st) and end (en) location of each estimated homogeneous region.
st <- output$estimates
en <- st[-1] - 1
# Obtain 2D-kde.
est1 <- kde2d(stppData[st[1]:en[1], 3], stppData[st[1]:en[1], 4], n = 250, lims = rep(c(-15, 
  15), 2))
est2 <- kde2d(stppData[st[2]:en[2], 3], stppData[st[2]:en[2], 4], n = 250, lims = rep(c(-15, 
  15), 2))
est3 <- kde2d(stppData[st[3]:en[3], 3], stppData[st[3]:en[3], 4], n = 250, lims = rep(c(-15, 
  15), 2))
est4 <- kde2d(stppData[st[4]:en[4], 3], stppData[st[4]:en[4], 4], n = 250, lims = rep(c(-15, 
  15), 2))
# Generate plots for 2D-kde.
par(mfrow = c(2, 2))
image(est1, main = "Estimated density for period 1", cex.main = 1.4, cex.axis = 0.8, 
  col = RColorBrewer::brewer.pal(9, "PuBuGn"))
image(est2, main = "Estimated density for period 2", cex.main = 1.4, cex.axis = 0.8, 
  col = RColorBrewer::brewer.pal(9, "PuBuGn"))
image(est3, main = "Estimated density for period 3", cex.main = 1.4, cex.axis = 0.8, 
  col = RColorBrewer::brewer.pal(9, "PuBuGn"))
image(est4, main = "Estimated density for period 4", cex.main = 1.4, cex.axis = 0.8, 
  col = RColorBrewer::brewer.pal(9, "PuBuGn"))

# Run procedure with a penalty function.
pen <- function(cp) {
  -length(cp)
}
output2 <- e.agglo(X = stppData[, 3:4], member = member, alpha = 1, penalty = pen)

# Generate plots of goodness-of-fit statistic progression
par(mfrow = c(2, 1))
plot(rev(output$fit)[-1], ylab = "Value", main = "No penalty", t = "l", xlab = "Number of change points")
plot(rev(output2$fit)[-1], ylab = "Value", main = "Penalize on number of change points", 
  t = "l", xlab = "Number of change points")

###########################
# Analysis of financial dataset
library("ecp")
data("DJIA")

xIndex <- DJIA$index
xMarket <- DJIA$market
dates <- DJIA$dates

yIndex <- e.divisive(xIndex, R = 499)
yMarket <- e.divisive(xMarket, R = 499)

n1 <- floor(nrow(xIndex)/30)
n2 <- floor(nrow(xMarket)/30)
mem1 <- rep(1:n1, rep(30, n1))
mem2 <- rep(1:n2, rep(30, n2))
mem1 <- c(mem1, rep(n1 + 1, nrow(xIndex) - length(mem1)))
mem2 <- c(mem2, rep(n2 + 2, nrow(xMarket) - length(mem2)))

aggIndex <- e.agglo(xIndex, mem1, 1, function(x) -length(x)/15)
aggMarket <- e.agglo(xMarket, mem2, 1, function(x) -length(x)/15)

# For plotting reasons we reverse the time order of the time series. The
# resulting plot has more recent dates on the right side of the image. As a
# result we also reverse the dates, and adjust the estimated change points.

dates <- rev(dates)
x <- as.matrix(rev(xIndex))

l <- yIndex$estimates     # Original set of change points.
l <- 1141 - l             # Adjust change points to account for time reversal.
l <- l[-c(1, length(l))]  # Remove change points at the start and end of the time series.

# Generate time series plot and vertical lines for estimated change points.

At <- seq(1, length(dates), length.out = 13)
Lab <- dates[At]

plot(x, t = "l", main = "Dow Jones Industrial Average Index", xlab = "Dates", ylab = "Weekly Log Return", 
  axes = F, cex.lab = 1, cex.main = 1)
box()
axis(1, at = At, labels = Lab, cex.axis = 0.8)

At2 <- seq(-0.1, 0.1, length.out = 5)
Lab2 <- as.character(At2)
axis(2, at = At2, labels = Lab2, cex.axis = 0.8)

abline(v = l, col = 2, lty = 2)


########################
# Analysis of genetics dataset
# The inline and Rcpp packages are needed to use the MultiRankCpp.R file


library("ecp")
source("MultiRankCpp.R")
data(ACGH)

# Individuals 10 and 15 are columns 8 and 10 repsectively
person10 <- ACGH$data[, 8]
person15 <- ACGH$data[, 10]

# Obtain initial segmentation used by the E-Agglo method
S <- ceiling(length(person10)/15)
mem <- rep(1:S, rep(15, S))
mem <- mem[1:length(person10)]

# Perform analysis for individual 10
Ediv10 <- e.divisive(as.matrix(person10), R = 499, min.size = 15, alpha = 2)$estimates
Eagg10 <- e.agglo(as.matrix(person10), mem, alpha = 2)$estimates
Mran10 <- MultiRank(t(person10), 147)$list
# Results for GLF method are documented in the README file
GFL10 <- c(134, 177, 393, 529, 540, 578, 656, 788, 811, 869, 932, 960, 1051, 1141, 
  1285, 1319, 1367, 1560, 1656, 1659, 1724, 1906, 1972, 1997, 2041, 2141, 2200, 
  2215)

# Perform analysis for individual 15
Ediv15 <- e.divisive(as.matrix(person15), R = 499, min.size = 15, alpha = 2)$estimates
Eagg15 <- e.agglo(as.matrix(person15), mem, alpha = 2)$estimates
Mran15 <- MultiRank(t(person15), 147)$list
# Results for GFL method are documented in the README file
GFL15 <- c(72, 134, 428, 450, 775, 811, 1119, 1141, 1215, 1321, 1367, 1455, 1534, 
  1560, 1642, 1685, 1726, 1816, 1991, 1992, 2047, 2091, 2143, 2168, 2201, 2209, 
  2210, 2213, 2215)

# Remove the identified change point corresponding to the beginning and end of
# the time series
Ediv10 <- Ediv10[-c(1, length(Ediv10))]
Ediv15 <- Ediv15[-c(1, length(Ediv15))]
Mran10 <- Mran10[-c(1, length(Mran10))]
Mran15 <- Mran15[-c(1, length(Mran15))]
Eagg10 <- Eagg10[-c(1, length(Eagg10))]
Eagg15 <- Eagg15[-c(1, length(Eagg15))]

# Generaete plots
library("ggplot2")
# Create data.frame for original data
Points10 <- data.frame(index = 1:length(person10), Signal = person10, group = "GFL")
Points10 <- rbind(Points10, data.frame(index = 1:length(person10), Signal = person10, 
  group = "E-Divisive"))
Points10 <- rbind(Points10, data.frame(index = 1:length(person10), Signal = person10, 
  group = "E-Agglo"))
Points10 <- rbind(Points10, data.frame(index = 1:length(person10), Signal = person10, 
  group = "MultiRank"))

# Create data.frame for set of change points
breaks10 <- data.frame(changepoints = GFL10, group = "GFL")
breaks10 <- rbind(breaks10, data.frame(changepoints = Ediv10, group = "E-Divisive"))
breaks10 <- rbind(breaks10, data.frame(changepoints = Eagg10, group = "E-Agglo"))
breaks10 <- rbind(breaks10, data.frame(changepoints = Mran10, group = "MultiRank"))
# Display plots for Individual 10
g10 <- ggplot(Points10, aes(index, Signal)) + geom_point() + theme_bw() + theme(panel.grid.major = element_blank(), 
  panel.grid.minor = element_blank())
g10 <- g10 + facet_grid(group ~ .)
g10 <- g10 + geom_vline(data = breaks10, aes(xintercept = changepoints, color = group), 
  linetype = "dashed")
g10 <- g10 + ggtitle("Individual 10")

print(g10)

# Create data.frame for original data
Points15 <- data.frame(index = 1:length(person15), Signal = person15, group = "GFL")
Points15 <- rbind(Points15, data.frame(index = 1:length(person15), Signal = person15, 
  group = "E-Divisive"))
Points15 <- rbind(Points15, data.frame(index = 1:length(person15), Signal = person15, 
  group = "E-Agglo"))
Points15 <- rbind(Points15, data.frame(index = 1:length(person15), Signal = person15, 
  group = "MultiRank"))

# Create data.frame for set of change points
breaks15 <- data.frame(changepoints = GFL15, group = "GFL")
breaks15 <- rbind(breaks15, data.frame(changepoints = Ediv15, group = "E-Divisive"))
breaks15 <- rbind(breaks15, data.frame(changepoints = Eagg15, group = "E-Agglo"))
breaks15 <- rbind(breaks15, data.frame(changepoints = Mran15, group = "MultiRank"))
# Display plots for Individual 15
g15 <- ggplot(Points15, aes(index, Signal)) + geom_point() + theme_bw() + theme(panel.grid.major = element_blank(), 
  panel.grid.minor = element_blank())
g15 <- g15 + facet_grid(group ~ .)
g15 <- g15 + geom_vline(data = breaks15, aes(xintercept = changepoints, color = group), 
  linetype = "dashed")
g15 <- g15 + ggtitle("Individual 15")

print(g15) 
